/*==============================================================================


       main.cpp - addon file to main


==============================================================================*/
#include "sys.h"         // eBookMan include files
#include "gui.h"

extern "C"
{
#include "ereader_hostio.h"
#include "evnt_fun.h"
#include "lcd.h"
#include "timer.h"
#include "ereader_hostio.h"
#include "piezo.h"
#include "ebm_object.h"
#include "stdlib.h"
}


#include "main_fff.h"
#include "enter_name_fff.h"
#include "setup_fff.h"

char *Help_text = "Garbage is a card game where you place the correct answers in the multiplication table.
Select your card from the garbage or pick pile.  Start new game and setup are in the menu.
Jog wheel will toggle sound on and off.
Have fun.
********************
WhitWareInc Software [whitwareinc@yahoo.com]
Version 2.01
"__DATE__"-"__TIME__"
";

#define TIMER_SPEED	500	/* currently not used */
#define RND_TIMER	500

#define MAX_PLAYERS	6
#define MAX_MATRIX	12
#define MAX_COLUMNS	10
#define MAX_ROWS	10

enum {
	MSG_GAME_LOOP=MSG_USER+1,
};

enum {
	GS_PICK = 1,
	GS_PLACE,
	GS_NEXT,
};

MainWindow *f_MainWindow;
EnterNameWindow *f_EnterNameWindow;
SetupWindow *f_SetupWindow;

U16 Game_score, Pick_card, Got_garbage_card, Garbage_card, Num_cards, Card_num, Num_turns;
U8 Num_rows, Num_columns, Num_deck_cards;

U8 Running;
U8 Game_state, Current_player, Num_players;
CLabel *Table_obj[MAX_MATRIX][MAX_MATRIX];
char Table_obj_str[MAX_PLAYERS][MAX_MATRIX][MAX_MATRIX][4];
char Player_names[MAX_PLAYERS][50];
U16 Player_card_filled[MAX_PLAYERS];
char Current_name_str[50];
char Pick_button_str[20];
char Garbage_button_str[20];
char Players_button_str[20];
char Columns_button_str[20];
char Rows_button_str[20];
char You_won_str[50];

U8 Sound;

U16 Deck[MAX_MATRIX*MAX_MATRIX*MAX_PLAYERS];

typedef struct score_struct {U16 score; char name[50];};

void MainWindow::HighScore (U16 score, char *name_p)
{
	struct score_struct *high_scores, tmp_score;
	size_t max_size=ebo_roundup(sizeof(score_struct)*10);

	static const ebo_name_t file_name = { "","WhitWare","Garbage","ini" };
	char *file_ptr=(char *)ebo_roundup(OS_availaddr);
	OS_availaddr=(U32)file_ptr + max_size;

	U16 num_scores, i, j;
	char high_scores_str[500];
	
	if (ebo_mapin(&file_name,0,file_ptr,&max_size,1) < 0)
	{
		if ( ebo_new(&file_name,max_size,1) < 0)
      			return;
		else if (ebo_mapin(&file_name,0,file_ptr,&max_size,1) < 0)
			return;
	}
	high_scores = (score_struct *)file_ptr;
	num_scores = 0;
	high_scores_str[0] = 0;
	while ( high_scores[num_scores].score )
	{
		num_scores++;
	}

	if(score)
	{
		if(num_scores > 9)
			num_scores = 9;
		high_scores[num_scores].score = score;
		strcpy(high_scores[num_scores].name, name_p);
		SetFocus(f_MainWindow);
		num_scores++;
	}
	// qsort(high_scores, (size_t)num_scores, (size_t)sizeof(score_struct), (int (*)(...))ScoreCmp);
	for (i = 0; i < num_scores; i++)
	{
		for (j = i+1; j < num_scores; j++)
			if(high_scores[j].score > high_scores[i].score)
			{
				tmp_score = high_scores[i];
				high_scores[i] = high_scores[j];
				high_scores[j] = tmp_score;
			}
	}		
	sprintf(&high_scores_str[strlen(high_scores_str)],"*************\n  High Scores\n*************\n");
	for (i = 0; i < num_scores; i++)
	{
		sprintf(&high_scores_str[strlen(high_scores_str)],"%6d | %s\n",
			high_scores[i].score, high_scores[i].name);
	}

	GUI_Alert(ALERT_INFO, high_scores_str);

	ebo_unmap(file_ptr, max_size);
}


void MainWindow::NewGame ()
{
	U8 x, y, p;
	U16 Column_size;
	U16 shuffle, tmp, i;

	delete Board;
	Board=new CTable(MainWindow_Board,196,183);
	AddChild(Board,0,0);
		
	for (p = 0; p < Num_players; p++)
	{
		if(!p) 
			sprintf(Player_names[p], "eBookman ");
		else
			sprintf(Player_names[p], "Player %d ", p);
		Player_card_filled[p]=0;
	}

	Current_player = 0;
	Num_turns = 0;
	Column_size = 196/(Num_columns+1) - 3;
	for (x = 0; x < Num_columns; x++)
		Board->AddColumn(Column_size);
	Board->AddColumn(0);
	Board->SetNumRows(Num_rows+1);
	
	for (p = 0; p < Num_players; p++)
	{
		for (y = 0; y <= Num_rows; y++)
		{
			for (x = 0; x <= Num_columns; x++)
			{
				if(!y && x)
				{
					sprintf(Table_obj_str[p][y][x], "%d", x);
				}
				else if(!x && y)
				{
					sprintf(Table_obj_str[p][y][x], "%d", y);
				}
				else
				{
					sprintf(Table_obj_str[p][y][x], "*");
				}
			}
		}
	}
	for (y = 0; y <= Num_rows; y++)
	{
		for (x = 0; x <= Num_columns; x++)
		{
			Table_obj[y][x]=new CLabel(MainWindow_Board,"",0);
			Board->SetCell( Table_obj[y][x], y, x );
			Table_obj[y][x]->SetLabel(Table_obj_str[Current_player][y][x]);
		}
	}

	Num_cards = Num_rows * Num_columns;
	Num_deck_cards = 0;
	for (i = 0; i < Num_players; i++)
		for (y = 1; y <= Num_rows; y++)
			for (x = 1; x <= Num_columns; x++)
				Deck[Num_deck_cards++] = y*x;
	for (x = 0; x < MAX_PLAYERS; x++)
		for (i = 0; i < Num_deck_cards; i++)
		{
			shuffle = rand()%(Num_deck_cards);
			tmp = Deck[i];
			Deck[i] = Deck[shuffle];
			Deck[shuffle] = tmp;
		}
	Card_num = 0;
	sprintf(Current_name_str, "%s-pick/garbage", Player_names[Current_player]);
	CurrentName->SetLabel(Current_name_str);
	Game_state = GS_PICK;
	Running = 1;
	Board->Draw();
	GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow->Board, 0 );
}
//------------------------------------------------------------------------------
void SetupWindow::Close( void )
{
	f_MainWindow->SetFocus(f_MainWindow);
	return CWindow::Close();
}
			

//------------------------------------------------------------------------------
S32 SetupWindow::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
	switch ( type )
	{
		case MSG_BUTTON_SELECT:
			switch( from->GetID() )
			{
				case SetupWindow_PlayersButton:
					if(++Num_players > MAX_PLAYERS)
						Num_players = 2;
					sprintf(Players_button_str, "Players - %d", Num_players);
					PlayersButton->SetLabel(Players_button_str);
					break;
				
				case SetupWindow_ColumnsButton:
					if(++Num_columns > MAX_COLUMNS)
						Num_columns = 5;
					sprintf(Columns_button_str, "Colimns - %d", Num_columns);
					ColumnsButton->SetLabel(Columns_button_str);
					break;
				
				case SetupWindow_RowsButton:
					if(++Num_rows > MAX_ROWS)
						Num_rows = 2;
					sprintf(Rows_button_str, "Rows - %d", Num_rows);
					RowsButton->SetLabel(Rows_button_str);
					break;
				
				default:
					return 0;
			}
			return 1;
		default:
			return 0;
	}
	return CWindow::MsgHandler(type, from, data);
}

//------------------------------------------------------------------------------
void EnterNameWindow::Close( void )
{
	char *name_p;

	name_p = f_EnterNameWindow->EnterName->GetText();
	f_MainWindow->HighScore(Num_cards*10-Num_turns, name_p);
	free(name_p);

	return CWindow::Close();
}

			

//------------------------------------------------------------------------------
S32 MainWindow::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
U16 x, y;
S32 new_data;

 // message handler

	switch ( type )
	{
		case MSG_KEY:
			switch ( data )
			{
				case K_MENU: // process request to show menu
					m_menu1->Show();
					return 1;
				case K_JOG_UP:
				case K_JOG_ENTER:
				case K_JOG_DOWN:
					Sound ^= 1;
					return 1;
				default:
					return 0;
			}
		case MSG_PEN_DOWN:
            		if (Running==1)
            		{
				if(Sound)
				{
					piezo_play_jingle(6, PIEZO_FLAG_NOQUIET|PIEZO_FLAG_WAIT);
				}
				if (Game_state == GS_PLACE)
				{
					y = Board->FindRowFromY(data & 0xffff);
					x = Board->FindColFromX((data >> 16) & 0xffff);
					if ((x != 65535) && (y != 65535))
					{
						if (((x*y) == Pick_card) && (Table_obj_str[Current_player][y][x][0]=='*'))
						{
							sprintf(Table_obj_str[Current_player][y][x], "%d", y*x);
							Table_obj[y][x]->SetLabel(Table_obj_str[Current_player][y][x]);
							if(++Player_card_filled[Current_player] >= Num_cards)
							{
								Running = 0;
								GUI_UpdateNow();
								sprintf(You_won_str, "%s won!", Player_names[Current_player]);
								GUI_Alert(ALERT_INFO, You_won_str);
								if(!Current_player)
								{
									HighScore(Num_cards*10-Num_turns, Player_names[Current_player]);
								}
								else
								{
									f_EnterNameWindow = new EnterNameWindow();
								}
							}
							Pick_card = Deck[Card_num++%(Num_deck_cards)];
							sprintf(Pick_button_str, "Pick-%d", Pick_card);
							PickButton->SetLabel(Pick_button_str);
						}
						else
						{
							sprintf(Pick_button_str, "Pick");
							PickButton->SetLabel(Pick_button_str);
							Got_garbage_card=1;
							Garbage_card = Pick_card;
							sprintf(Garbage_button_str, "Garb-%d", Pick_card);
							GarbageButton->SetLabel(Garbage_button_str);
 							sprintf(Current_name_str, "%s-Next", Player_names[Current_player]);
							CurrentName->SetLabel(Current_name_str);
							Game_state++;
						}
					}
				}
			}
			return 1;
 			
            	case MSG_TIMER:
            		GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), from, 0 );
            		return 1;
            		
            	case MSG_GAME_LOOP:
            		if (Running==1)
            		{
            			do
            			{
					if(Game_state == GS_PICK)
					{
						U8 got_one = 0;
						if (Got_garbage_card)					
							for (y = 0; ((y <= Num_rows) && (!got_one)); y++)
								for (x = 0; ((x <= Num_columns) && (!got_one)); x++)
									if(((x*y) == Garbage_card) && (Table_obj_str[Current_player][y][x][0]=='*'))
									{
			            						got_one++;
										from->SetID(MainWindow_GarbageButton);
			            						GUI_EventMessage( (MSG_TYPE)(MSG_BUTTON_REPEAT), from, 0 );
									}
						if (!got_one)
						{
							from->SetID(MainWindow_PickButton);
			            			GUI_EventMessage( (MSG_TYPE)(MSG_BUTTON_REPEAT), from, 0 );
				            	}
					}
					else if(Game_state == GS_PLACE)
					{
						U8 got_one = 0;
						for (y = 0; ((y <= Num_rows) && (!got_one)); y++)
							for (x = 0; ((x <= Num_columns) && (!got_one)); x++)
								if(((x*y) == Pick_card) && (Table_obj_str[Current_player][y][x][0]=='*'))
								{
									U32 xw =Board->GetColumnWidth(0)+3;
									U32 yw =Board->RowHeight(0)+3;
									new_data = (((x*xw)+(xw/2)) << 16) | ((y*yw)+(yw/2)); 	
			            					GUI_EventMessage( (MSG_TYPE)(MSG_PEN_DOWN), from, new_data );
			            					got_one++;
								}
						if (!got_one)
						{
							from->SetID(MainWindow_GarbageButton);
				            		GUI_EventMessage( (MSG_TYPE)(MSG_BUTTON_REPEAT), from, 0 );
				            	}
					}
					GUI_EventLoop(0);
				} while (Game_state != GS_NEXT);
			}
			else
			{
				Board->SetTimer(RND_TIMER, FALSE, 0);
			}
			rand();
 			return 1;

		case MSG_BUTTON_REPEAT:
		case MSG_BUTTON_SELECT:
			if (Running)
			{
				switch( from->GetID() )
				{
					case MainWindow_PickButton:
						if(Game_state == GS_PICK)
						{
							Pick_card = Deck[Card_num++%(Num_deck_cards)];
							sprintf(Pick_button_str, "Pick-%d", Pick_card);
							PickButton->SetLabel(Pick_button_str);
 							sprintf(Current_name_str, "%s-place/garbage", Player_names[Current_player]);
							CurrentName->SetLabel(Current_name_str);
							Game_state++;
						}
						break;
					case MainWindow_GarbageButton:
						if(Game_state == GS_PICK)
						{
							if(Got_garbage_card)
							{
								Got_garbage_card=0;
								Pick_card = Garbage_card;
								sprintf(Pick_button_str, "Pick-%d", Pick_card);
								PickButton->SetLabel(Pick_button_str);
								sprintf(Garbage_button_str, "garbage");
								GarbageButton->SetLabel(Garbage_button_str);
 								sprintf(Current_name_str, "%s-place/garbage", Player_names[Current_player]);
								CurrentName->SetLabel(Current_name_str);
								Game_state++;
							}
						}
						else if (Game_state == GS_PLACE)
						{
							sprintf(Pick_button_str, "Pick");
							PickButton->SetLabel(Pick_button_str);
							Got_garbage_card=1;
							Garbage_card = Pick_card;
							sprintf(Garbage_button_str, "Garb-%d", Pick_card);
							GarbageButton->SetLabel(Garbage_button_str);
 							sprintf(Current_name_str, "%s-Next", Player_names[Current_player]);
							CurrentName->SetLabel(Current_name_str);
							Game_state++;
						}
						break;
					case MainWindow_NextButton:
						if(Game_state == GS_NEXT)
						{
							Current_player = ++Current_player % Num_players;
							for (y = 0; y <= Num_rows; y++)
								for (x = 0; x <= Num_columns; x++)
									Table_obj[y][x]->SetLabel(Table_obj_str[Current_player][y][x]);
 							sprintf(Current_name_str, "%s-pick/garbage", Player_names[Current_player]);
							CurrentName->SetLabel(Current_name_str);
							Game_state = GS_PICK;

							if(!Current_player)
							{
								Num_turns++;
								GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow->Board, 0 );
            						}
						}
						else if (Game_state == GS_PLACE)
						{
							sprintf(Pick_button_str, "Pick");
							PickButton->SetLabel(Pick_button_str);
							Got_garbage_card=1;
							Garbage_card = Pick_card;
							sprintf(Garbage_button_str, "Garb-%d", Pick_card);
							GarbageButton->SetLabel(Garbage_button_str);
							Current_player = ++Current_player % Num_players;
							for (y = 0; y <= Num_rows; y++)
								for (x = 0; x <= Num_columns; x++)
									Table_obj[y][x]->SetLabel(Table_obj_str[Current_player][y][x]);
 							sprintf(Current_name_str, "%s-pick/garbage", Player_names[Current_player]);
							CurrentName->SetLabel(Current_name_str);
							Game_state = GS_PICK;
							if(!Current_player)
							{
								Num_turns++;
								GUI_EventMessage( (MSG_TYPE)(MSG_GAME_LOOP), f_MainWindow->Board, 0 );
            						}
						}
						break;
					default:
						return 0;
				}
  				return (1);
			}
			switch( from->GetID() )
			{
				case MainWindow_NextButton:
					Current_player = ++Current_player % Num_players;
					for (y = 0; y <= Num_rows; y++)
						for (x = 0; x <= Num_columns; x++)
							Table_obj[y][x]->SetLabel(Table_obj_str[Current_player][y][x]);
					sprintf(Current_name_str, "%s-Board", Player_names[Current_player]);
					CurrentName->SetLabel(Current_name_str);
					break;
				default:
					return 0;
			}
  			return (1);

		case MSG_MENU_SELECT: // handle menu selection
		{
			switch ((U16)data)
			{
				case IDM_OPTION1: // New Game
					NewGame();
					break;
				case IDM_OPTION2: // High scores
					HighScore(0, 0);
					break;
				case IDM_OPTION3: // Help
					GUI_TextWindow(1,"Help",Help_text);
					break;
				case IDM_OPTION4: // Handwriting help
					GUI_HandwritingHelp(0);
					break;
				case IDM_OPTION5: // Setup
					f_SetupWindow = new SetupWindow();
					sprintf(Players_button_str, "Players - %d", Num_players);
					f_SetupWindow->PlayersButton->SetLabel(Players_button_str);
					sprintf(Columns_button_str, "Colimns - %d", Num_columns);
					f_SetupWindow->ColumnsButton->SetLabel(Columns_button_str);
					sprintf(Rows_button_str, "Rows - %d", Num_rows);
					f_SetupWindow->RowsButton->SetLabel(Rows_button_str);
					break;
				case IDM_OPTION6: // Exit
					GUI_Exit();
					break;

			}
			return 1;
		}

		default:
			return 0;
	}
				
  return CWindow::MsgHandler(type, from, data);
}

//------------------------------------------------------------------------------
void MainWindow::Initialize(void)
{
// set up menu #1 - with shortcuts
	m_menu1=new CMenu(ID_MENU1,0,32767,0,this);
	m_menu1->SetNumRows(7);
	int row=-1;
	m_menu1->SetRow( ++row,IDM_OPTION1,"New Game",'n' );
	m_menu1->SetRow( ++row,IDM_OPTION2,"High Scores",'s' );
	m_menu1->SetRow( ++row,IDM_OPTION3,"Help",'h' );
	m_menu1->SetRow( ++row,IDM_OPTION4,"Handwriting Help",'w' );
	m_menu1->SetSeparatorRow(++row);
	m_menu1->SetRow( ++row,IDM_OPTION5,"Setup");
	m_menu1->SetRow( ++row,IDM_OPTION6,"Exit",'e' );
	
//	m_menuBar=new CMenuBar(ID_MENUBAR,23,this);
//	m_menuBar->AddButton(new CPushButton( ID_MENU1_BUTTON,0,0,"Menu"),m_menu1 );

// do additional setup functions

	Num_rows = 5;
	Num_columns = 5;
	Num_players = 2;

	EnableOSNotify();
	rand();
	Board->SetTimer(RND_TIMER, FALSE, 0);

	Sound = 1;
}


//------------------------------------------------------------------------------
MainWindow::~MainWindow(void)
{
	//delete m_menu1; // cleanup
}

//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
	switch (type)
	{
	case MSG_APP_START:
		if (OS_is_present && hostIO_am_I_the_current_task())
			HOSTIO_INLINE_BREAKPOINT();
		f_MainWindow = new MainWindow();
		f_MainWindow->Initialize(); // do additional setup here
		return 1;
	case MSG_APP_STOP: // prog exit & cleanup
		//delete f_MainWindow;
		return 1;
	case MSG_OS_NOTIFY:
		switch ((U16) data)
		{
			case NTFY_POWER_ON:
				return 0;
			case NTFY_FOREGROUND:
				if(Running == 2)
					Running = 1;
				return 1;
			case NTFY_BACKGROUND:
				if(Running)
					Running = 2;
				return 1;
			default:
				return 0;
		}
	default:
		break;
	}
	return 0;
};
//------------------------------------------------------------------------------


